@@ -1,17 +1,32 @@
 # functions to support bundled libraries
 
-from Configure import conf
-import sys, Logs
-from samba_utils import *
+import sys
+from waflib import Build, Options, Logs
+from waflib.Configure import conf
+from wafsamba import samba_utils
 
 def PRIVATE_NAME(bld, name, private_extension, private_library):
     '''possibly rename a library to include a bundled extension'''
 
+    if not private_library:
+        return name
+
     # we now use the same private name for libraries as the public name.
     # see http://git.samba.org/?p=tridge/junkcode.git;a=tree;f=shlib for a
     # demonstration that this is the right thing to do
     # also see http://lists.samba.org/archive/samba-technical/2011-January/075816.html
-    return name
+    if private_extension:
+        return name
+
+    extension = bld.env.PRIVATE_EXTENSION
+
+    if extension and name.startswith('%s' % extension):
+        return name
+
+    if extension and name.endswith('%s' % extension):
+        return name
+
+    return "%s-%s" % (name, extension)
 
 
 def target_in_list(target, lst, default):
@@ -36,19 +51,19 @@ Build.BuildContext.BUILTIN_LIBRARY = BUILTIN_LIBRARY
 
 def BUILTIN_DEFAULT(opt, builtins):
     '''set a comma separated default list of builtin libraries for this package'''
-    if 'BUILTIN_LIBRARIES_DEFAULT' in Options.options:
+    if 'BUILTIN_LIBRARIES_DEFAULT' in Options.options.__dict__:
         return
-    Options.options['BUILTIN_LIBRARIES_DEFAULT'] = builtins
-Options.Handler.BUILTIN_DEFAULT = BUILTIN_DEFAULT
+    Options.options.__dict__['BUILTIN_LIBRARIES_DEFAULT'] = builtins
+Options.OptionsContext.BUILTIN_DEFAULT = BUILTIN_DEFAULT
 
 
 def PRIVATE_EXTENSION_DEFAULT(opt, extension, noextension=''):
     '''set a default private library extension'''
-    if 'PRIVATE_EXTENSION_DEFAULT' in Options.options:
+    if 'PRIVATE_EXTENSION_DEFAULT' in Options.options.__dict__:
         return
-    Options.options['PRIVATE_EXTENSION_DEFAULT'] = extension
-    Options.options['PRIVATE_EXTENSION_EXCEPTION'] = noextension
-Options.Handler.PRIVATE_EXTENSION_DEFAULT = PRIVATE_EXTENSION_DEFAULT
+    Options.options.__dict__['PRIVATE_EXTENSION_DEFAULT'] = extension
+    Options.options.__dict__['PRIVATE_EXTENSION_EXCEPTION'] = noextension
+Options.OptionsContext.PRIVATE_EXTENSION_DEFAULT = PRIVATE_EXTENSION_DEFAULT
 
 
 def minimum_library_version(conf, libname, default):
@@ -70,139 +85,146 @@ def minimum_library_version(conf, libname, default):
 
 @conf
 def LIB_MAY_BE_BUNDLED(conf, libname):
-    return ('NONE' not in conf.env.BUNDLED_LIBS and
-            '!%s' % libname not in conf.env.BUNDLED_LIBS)
-
+    if libname in conf.env.SYSTEM_LIBS:
+        return False
+    if libname in conf.env.BUNDLED_LIBS:
+        return True
+    if '!%s' % libname in conf.env.BUNDLED_LIBS:
+        return False
+    if 'NONE' in conf.env.BUNDLED_LIBS:
+        return False
+    return True
 
 @conf
 def LIB_MUST_BE_BUNDLED(conf, libname):
-    return ('ALL' in conf.env.BUNDLED_LIBS or
-            libname in conf.env.BUNDLED_LIBS)
+    if libname in conf.env.BUNDLED_LIBS:
+        return True
+    if '!%s' % libname in conf.env.BUNDLED_LIBS:
+        return False
+    if 'ALL' in conf.env.BUNDLED_LIBS:
+        return True
+    return False
 
 @conf
 def LIB_MUST_BE_PRIVATE(conf, libname):
     return ('ALL' in conf.env.PRIVATE_LIBS or
             libname in conf.env.PRIVATE_LIBS)
 
-@conf
-def CHECK_PREREQUISITES(conf, prereqs):
-    missing = []
-    for syslib in TO_LIST(prereqs):
-        f = 'FOUND_SYSTEMLIB_%s' % syslib
-        if not f in conf.env:
-            missing.append(syslib)
-    return missing
-
-
-@runonce
 @conf
 def CHECK_BUNDLED_SYSTEM_PKG(conf, libname, minversion='0.0.0',
+        maxversion=None, version_blacklist=[],
         onlyif=None, implied_deps=None, pkg=None):
     '''check if a library is available as a system library.
 
     This only tries using pkg-config
     '''
-    if conf.LIB_MUST_BE_BUNDLED(libname):
-        return False
-    found = 'FOUND_SYSTEMLIB_%s' % libname
-    if found in conf.env:
-        return conf.env[found]
+    return conf.CHECK_BUNDLED_SYSTEM(libname,
+                                     minversion=minversion,
+                                     maxversion=maxversion,
+                                     version_blacklist=version_blacklist,
+                                     onlyif=onlyif,
+                                     implied_deps=implied_deps,
+                                     pkg=pkg)
 
-    # see if the library should only use a system version if another dependent
-    # system version is found. That prevents possible use of mixed library
-    # versions
-    if onlyif:
-        missing = conf.CHECK_PREREQUISITES(onlyif)
-        if missing:
-            if not conf.LIB_MAY_BE_BUNDLED(libname):
-                Logs.error('ERROR: Use of system library %s depends on missing system library/libraries %r' % (libname, missing))
-                sys.exit(1)
-            conf.env[found] = False
-            return False
-
-    minversion = minimum_library_version(conf, libname, minversion)
-
-    msg = 'Checking for system %s' % libname
-    if minversion != '0.0.0':
-        msg += ' >= %s' % minversion
-
-    if pkg is None:
-        pkg = libname
-
-    if conf.check_cfg(package=pkg,
-                      args='"%s >= %s" --cflags --libs' % (pkg, minversion),
-                      msg=msg, uselib_store=libname.upper()):
-        conf.SET_TARGET_TYPE(libname, 'SYSLIB')
-        conf.env[found] = True
-        if implied_deps:
-            conf.SET_SYSLIB_DEPS(libname, implied_deps)
-        return True
-    conf.env[found] = False
-    if not conf.LIB_MAY_BE_BUNDLED(libname):
-        Logs.error('ERROR: System library %s of version %s not found, and bundling disabled' % (libname, minversion))
-        sys.exit(1)
-    return False
-
-
-@runonce
 @conf
 def CHECK_BUNDLED_SYSTEM(conf, libname, minversion='0.0.0',
-                         checkfunctions=None, headers=None,
+                         maxversion=None, version_blacklist=[],
+                         checkfunctions=None, headers=None, checkcode=None,
                          onlyif=None, implied_deps=None,
-                         require_headers=True):
+                         require_headers=True, pkg=None, set_target=True):
     '''check if a library is available as a system library.
     this first tries via pkg-config, then if that fails
     tries by testing for a specified function in the specified lib
     '''
-    if conf.LIB_MUST_BE_BUNDLED(libname):
-        return False
+    # We always do a logic validation of 'onlyif' first
+    missing = []
+    if onlyif:
+        for l in samba_utils.TO_LIST(onlyif):
+            f = 'FOUND_SYSTEMLIB_%s' % l
+            if not f in conf.env:
+                Logs.error('ERROR: CHECK_BUNDLED_SYSTEM(%s) - ' % (libname) +
+                           'missing prerequisite check for ' +
+                           'system library %s, onlyif=%r' % (l, onlyif))
+                sys.exit(1)
+            if not conf.env[f]:
+                missing.append(l)
     found = 'FOUND_SYSTEMLIB_%s' % libname
     if found in conf.env:
         return conf.env[found]
-
-    def check_functions_headers():
-        '''helper function for CHECK_BUNDLED_SYSTEM'''
-        if checkfunctions is None:
-            return True
-        if require_headers and headers and not conf.CHECK_HEADERS(headers, lib=libname):
-            return False
-        return conf.CHECK_FUNCS_IN(checkfunctions, libname, headers=headers,
-                                   empty_decl=False, set_target=False)
+    if conf.LIB_MUST_BE_BUNDLED(libname):
+        conf.env[found] = False
+        return False
 
     # see if the library should only use a system version if another dependent
     # system version is found. That prevents possible use of mixed library
     # versions
-    if onlyif:
-        missing = conf.CHECK_PREREQUISITES(onlyif)
-        if missing:
-            if not conf.LIB_MAY_BE_BUNDLED(libname):
-                Logs.error('ERROR: Use of system library %s depends on missing system library/libraries %r' % (libname, missing))
-                sys.exit(1)
-            conf.env[found] = False
+    if missing:
+        if not conf.LIB_MAY_BE_BUNDLED(libname):
+            Logs.error('ERROR: Use of system library %s depends on missing system library/libraries %r' % (libname, missing))
+            sys.exit(1)
+        conf.env[found] = False
+        return False
+
+    def check_functions_headers_code():
+        '''helper function for CHECK_BUNDLED_SYSTEM'''
+        if require_headers and headers and not conf.CHECK_HEADERS(headers, lib=libname):
             return False
+        if checkfunctions is not None:
+            ok = conf.CHECK_FUNCS_IN(checkfunctions, libname, headers=headers,
+                                     empty_decl=False, set_target=False)
+            if not ok:
+                return False
+        if checkcode is not None:
+            define='CHECK_BUNDLED_SYSTEM_%s' % libname.upper()
+            ok = conf.CHECK_CODE(checkcode, lib=libname,
+                                 headers=headers, local_include=False,
+                                 msg=msg, define=define)
+            conf.CONFIG_RESET(define)
+            if not ok:
+                return False
+        return True
 
     minversion = minimum_library_version(conf, libname, minversion)
 
     msg = 'Checking for system %s' % libname
+    msg_ver = []
     if minversion != '0.0.0':
-        msg += ' >= %s' % minversion
+        msg_ver.append('>=%s' % minversion)
+    if maxversion is not None:
+        msg_ver.append('<=%s' % maxversion)
+    for v in version_blacklist:
+        msg_ver.append('!=%s' % v)
+    if msg_ver != []:
+        msg += " (%s)" % (" ".join(msg_ver))
+
+    uselib_store=libname.upper()
+    if pkg is None:
+        pkg = libname
+
+    version_checks = '%s >= %s' % (pkg, minversion)
+    if maxversion is not None:
+        version_checks += ' %s <= %s' % (pkg, maxversion)
+
+    version_checks += "".join(' %s != %s' % (pkg, v) for v in version_blacklist)
 
     # try pkgconfig first
-    if (conf.check_cfg(package=libname,
-                      args='"%s >= %s" --cflags --libs' % (libname, minversion),
-                      msg=msg) and
-        check_functions_headers()):
-        conf.SET_TARGET_TYPE(libname, 'SYSLIB')
+    if (conf.CHECK_CFG(package=pkg,
+                      args='"%s" --cflags --libs' % (version_checks),
+                      msg=msg, uselib_store=uselib_store) and
+        check_functions_headers_code()):
+        if set_target:
+            conf.SET_TARGET_TYPE(libname, 'SYSLIB')
         conf.env[found] = True
         if implied_deps:
             conf.SET_SYSLIB_DEPS(libname, implied_deps)
         return True
     if checkfunctions is not None:
-        if check_functions_headers():
+        if check_functions_headers_code():
             conf.env[found] = True
             if implied_deps:
                 conf.SET_SYSLIB_DEPS(libname, implied_deps)
-            conf.SET_TARGET_TYPE(libname, 'SYSLIB')
+            if set_target:
+                conf.SET_TARGET_TYPE(libname, 'SYSLIB')
             return True
     conf.env[found] = False
     if not conf.LIB_MAY_BE_BUNDLED(libname):
@@ -214,7 +236,6 @@ def CHECK_BUNDLED_SYSTEM(conf, libname, minversion='0.0.0',
 def tuplize_version(version):
     return tuple([int(x) for x in version.split(".")])
 
-@runonce
 @conf
 def CHECK_BUNDLED_SYSTEM_PYTHON(conf, libname, modulename, minversion='0.0.0'):
     '''check if a python module is available on the system and
