@@ -5,11 +5,14 @@ from waflib import Build, Logs, Utils, Configure, Errors
 from waflib.Configure import conf
 
 @conf
-def SAMBA_CHECK_PYTHON(conf, version=(3,5,0)):
+def SAMBA_CHECK_PYTHON(conf, version=(3,6,0)):
 
     if conf.env.disable_python:
         version=(2,6,0)
 
+    if conf.env.enable_fuzzing:
+        version=(3,5,0)
+
     # enable tool to build python extensions
     if conf.env.HAVE_PYTHON_H:
         conf.check_python_version(version)
